# Generated by Django 4.2.7 on 2023-12-05 18:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_form_runners", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="issue",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Issue",
                "verbose_name_plural": "Issues",
            },
        ),
        migrations.RemoveConstraint(
            model_name="issue",
            name="unique_label_lower_subject_identifier_etc",
        ),
        migrations.RemoveIndex(
            model_name="issue",
            name="edc_form_ru_label_l_ce3dda_idx",
        ),
        migrations.RemoveIndex(
            model_name="issue",
            name="edc_form_ru_subject_10bb16_idx",
        ),
        migrations.RenameField(
            model_name="issue",
            old_name="ignore_formfields",
            new_name="exclude_formfields",
        ),
        migrations.RemoveField(
            model_name="issue",
            name="status",
        ),
        migrations.AddField(
            model_name="issue",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="issue",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="issue",
            name="panel_name",
            field=models.CharField(max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="issue",
            name="schedule_name",
            field=models.CharField(max_length=25, null=True),
        ),
        migrations.AddField(
            model_name="issue",
            name="src_created_datetime",
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name="issue",
            name="src_user_created",
            field=models.CharField(max_length=150, null=True),
        ),
        migrations.AddField(
            model_name="issue",
            name="visit_schedule_name",
            field=models.CharField(max_length=25, null=True),
        ),
        migrations.AlterField(
            model_name="issue",
            name="label_lower",
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name="issue",
            name="subject_identifier",
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name="issue",
            name="visit_code",
            field=models.CharField(max_length=25),
        ),
        migrations.AlterField(
            model_name="issue",
            name="visit_code_sequence",
            field=models.IntegerField(),
        ),
        migrations.AddIndex(
            model_name="issue",
            index=models.Index(
                fields=["modified", "created"], name="edc_form_ru_modifie_486e31_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="issue",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_form_ru_user_mo_fd5ddf_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="issue",
            index=models.Index(
                fields=["label_lower", "field_name", "panel_name", "short_message"],
                name="edc_form_ru_label_l_c7917b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="issue",
            index=models.Index(
                fields=[
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                    "label_lower",
                    "panel_name",
                    "short_message",
                ],
                name="edc_form_ru_subject_6e8602_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="issue",
            constraint=models.UniqueConstraint(
                fields=(
                    "subject_identifier",
                    "label_lower",
                    "panel_name",
                    "visit_code",
                    "visit_code_sequence",
                    "visit_schedule_name",
                    "schedule_name",
                    "field_name",
                ),
                name="unique_label_lower_subject_identifier_etc",
            ),
        ),
    ]
